@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import abc
 import contextlib
 import hashlib
 
@@ -20,23 +19,8 @@ import six
 from keystonemiddleware.auth_token import _exceptions as exc
 from keystonemiddleware.auth_token import _memcache_crypt as memcache_crypt
 from keystonemiddleware.auth_token import _memcache_pool as memcache_pool
-from keystonemiddleware.i18n import _, _LE
-
-memcache = None  # module will be loaded on demand to avoid dependency
-
-
-def _create_memcache_client(*args, **kwargs):
-    """Create a new memcache client object.
-
-    This handles the lazy loaded import but also provides a point to mock out
-    in testing.
-    """
-    global memcache
-
-    if not memcache:
-        import memcache
-
-    return memcache.Client(*args, **kwargs)
+from keystonemiddleware.i18n import _, _LE, _LW
+from keystonemiddleware.openstack.common import memorycache
 
 
 def _hash_key(key):
@@ -70,8 +54,18 @@ class _EnvCachePool(object):
 class _CachePool(list):
     """A lazy pool of cache references."""
 
-    def __init__(self, memcached_servers):
+    def __init__(self, memcached_servers, log):
         self._memcached_servers = memcached_servers
+        if not self._memcached_servers:
+            log.warning(_LW(
+                "Using the in-process token cache is deprecated as of the "
+                "4.2.0 release and may be removed in the 5.0.0 release or "
+                "the 'O' development cycle. The in-process cache causes "
+                "inconsistent results and high memory usage. When the feature "
+                "is removed the auth_token middleware will not cache tokens "
+                "by default which may result in performance issues. It is "
+                "recommended to use  memcache for the auth_token token cache "
+                "by setting the memcached_servers option."))
 
     @contextlib.contextmanager
     def reserve(self):
@@ -79,7 +73,8 @@ class _CachePool(list):
         try:
             c = self.pop()
         except IndexError:
-            c = _create_memcache_client(self._memcached_servers, debug=0)
+            # the pool is empty, so we need to create a new client
+            c = memorycache.get_client(self._memcached_servers)
 
         try:
             yield c
@@ -87,59 +82,9 @@ class _CachePool(list):
             self.append(c)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class _CacheInterface(object):
-
-    def initialize(self, *args, **kwargs):
-        pass
-
-    @abc.abstractmethod
-    def store(self, key, value):
-        pass
-
-    @abc.abstractmethod
-    def store_invalid(self, key):
-        pass
-
-    @abc.abstractmethod
-    def get(self, key):
-        pass
-
-    def get_first(self, *args):
-        """Get the first cached value from many options.
-
-        :returns: token data if found else None.
-        """
-        for a in args:
-            value = self.get(a)
-
-            if value:
-                return value
-
-        return None
-
-
-class NoOpCache(_CacheInterface):
-
-    def store(self, key, value):
-        # Don't store anything
-        return None
-
-    def store_invalid(self, key):
-        # Don't store anything
-        return None
-
-    def get(self, key):
-        # Nothing to fetch from
-        return None
-
-    def get_first(self, *args):
-        # short circuit because calling get() multiple times wont help
-        return None
-
-
 class _MemcacheClientPool(object):
     """An advanced memcached client pool that is eventlet safe."""
+
     def __init__(self, memcache_servers, **kwargs):
         self._pool = memcache_pool.MemcacheClientPool(memcache_servers,
                                                       **kwargs)
@@ -150,7 +95,7 @@ class _MemcacheClientPool(object):
             yield client
 
 
-class TokenCache(_CacheInterface):
+class TokenCache(object):
     """Encapsulates the auth_token token cache functionality.
 
     auth_token caches tokens that it's seen so that when a token is re-used the
@@ -159,15 +104,13 @@ class TokenCache(_CacheInterface):
 
     initialize() must be called before calling the other methods.
 
-    Store a valid token in the cache using store(); mark a token as invalid in
-    the cache using store_invalid().
+    Store data in the cache store.
 
     Check if a token is in the cache and retrieve it using get().
 
     """
 
     _CACHE_KEY_TEMPLATE = 'tokens/%s'
-    _INVALID_INDICATOR = 'invalid'
 
     def __init__(self, log, cache_time=None,
                  env_cache_name=None, memcached_servers=None,
@@ -190,13 +133,8 @@ class TokenCache(_CacheInterface):
             return _MemcacheClientPool(self._memcached_servers,
                                        **self._memcache_pool_options)
 
-        elif self._memcached_servers:
-            return _CachePool(self._memcached_servers)
-
         else:
-            raise RuntimeError('Trying to configure a memcache cache without '
-                               'passing any servers. This should have been '
-                               'caught.')
+            return _CachePool(self._memcached_servers, self._LOG)
 
     def initialize(self, env):
         if self._initialized:
@@ -205,16 +143,6 @@ class TokenCache(_CacheInterface):
         self._cache_pool = self._get_cache_pool(env.get(self._env_cache_name))
         self._initialized = True
 
-    def store(self, token_id, data):
-        """Put token data into the cache."""
-        self._LOG.debug('Storing token in cache')
-        self._cache_store(token_id, data)
-
-    def store_invalid(self, token_id):
-        """Store invalid token in cache."""
-        self._LOG.debug('Marking token as unauthorized in cache')
-        self._cache_store(token_id, self._INVALID_INDICATOR)
-
     def _get_cache_key(self, token_id):
         """Get a unique key for this token id.
 
@@ -273,7 +201,6 @@ class TokenCache(_CacheInterface):
         If token is invalid raise exc.InvalidToken
         return token only if fresh (not expired).
         """
-
         if not token_id:
             # Nothing to do
             return
@@ -290,32 +217,13 @@ class TokenCache(_CacheInterface):
             serialized = serialized.encode('utf8')
         data = self._deserialize(serialized, context)
 
-        # Note that _INVALID_INDICATOR and (data, expires) are the only
-        # valid types of serialized cache entries, so there is not
-        # a collision with jsonutils.loads(serialized) == None.
         if not isinstance(data, six.string_types):
             data = data.decode('utf-8')
-        cached = jsonutils.loads(data)
-        if cached == self._INVALID_INDICATOR:
-            self._LOG.debug('Cached Token is marked unauthorized')
-            raise exc.InvalidToken(_('Token authorization failed'))
 
-        # NOTE(jamielennox): Cached values used to be stored as a tuple of data
-        # and expiry time. They no longer are but we have to allow some time to
-        # transition the old format so if it's a tuple just return the data.
-        try:
-            data, expires = cached
-        except ValueError:
-            data = cached
+        return jsonutils.loads(data)
 
-        return data
-
-    def _cache_store(self, token_id, data):
-        """Store value into memcache.
-
-        data may be _INVALID_INDICATOR or a tuple like (data, expires)
-
-        """
+    def set(self, token_id, data):
+        """Store value into memcache."""
         data = jsonutils.dumps(data)
         if isinstance(data, six.text_type):
             data = data.encode('utf-8')
